using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using UWBGL_XNA_Lib3;
using UWBGL_WinForms_Lib1;

namespace XNA_BallShoot
{
    public class Model
    {
        private Ball mHeroBall;
        private UWB_Array<Ball> mAllBalls;

        private int mSelectedIndex;

        private UWB_Clock mStopWatch;

        private float mWorldWidth;
        private float mWorldHeight;

        public Model()
        {
            mHeroBall = null;
            mAllBalls = new UWB_Array<Ball>();
            mWorldWidth = 0;
            mWorldHeight = 0;
            mSelectedIndex = -1;

            mStopWatch = new UWB_Clock();
        }

        public void setWorldSize(float width, float height)
        {
            mWorldWidth = width;
            mWorldHeight = height;
        }

        public void getWorldSize(ref float width, ref float height)
        {
            width = mWorldWidth;
            height = mWorldHeight;
        }

        public bool isDefiningHeroBall()
        {
            //if a heroBall exists it is being created
            return (mHeroBall != null);
        }

        public Ball selectedBall()
        {
            if (mHeroBall != null)
                return mHeroBall;

            if (mSelectedIndex < 0 || mSelectedIndex >= mAllBalls.count())
            {
                mSelectedIndex = -1;
                return null;
            }

            return mAllBalls.getItem(mSelectedIndex);
        }

        public int numBallsOnScreen()
        {
            int count = mAllBalls.count();
            if (mHeroBall != null)
                count++;
            return count;
        }

        public void createHeroBall(float worldX, float worldY)
        {
            if (mHeroBall != null)
            {
                mAllBalls.append(mHeroBall);
                mHeroBall = null;
            }

            mHeroBall = new Ball();
            mHeroBall.setCenter(worldX, worldY);
            mHeroBall.setVelocity(Vector3.Zero);
            mSelectedIndex = -1;
        }

        public void resizeHeroBall(float worldX, float worldY)
        {
            if(mHeroBall != null)
            {
                Vector3 center = mHeroBall.getCenter();
                float dx = worldX - center.X;
                float dy = worldY - center.Y;
                float radius = (float)Math.Sqrt(dx*dx + dy*dy);
                mHeroBall.setRadius(radius);

                Vector3 position = new Vector3(worldX, worldY, 0.0f);
                mHeroBall.setVelocity(mHeroBall.getCenter() - position);
            }
        }

        public void moveSelectedBallTo(float worldX, float worldY)
        {
            Ball ball = selectedBall();
            if (ball != null)
                ball.setCenter(worldX, worldY);
        }

        public void addHeroBallToWorld()
        {
            if (mHeroBall != null)
            {
                mAllBalls.append(mHeroBall);
                mHeroBall = null;
                mSelectedIndex = mAllBalls.count() - 1;
            }
        }

        public int selectBallAt(float worldX, float worldY)
        {
            mSelectedIndex = -1;

            for (int i = 0; i < mAllBalls.count(); i++)
            {
                Ball ball = mAllBalls.getItem(i);
                float radius = ball.getRadius();

                Vector3 center = ball.getCenter();
                float dx = worldX - center.X;
                float dy = worldY - center.Y;
                float distance = (float)Math.Sqrt(dx * dx + dy * dy);

                if (distance < radius)
                {
                    mSelectedIndex = i;
                    break;
                }
            }
            return mSelectedIndex;
        }

        public void updateSimulation()
        {
            float elapsedSeconds = mStopWatch.getSecondsElapsed();

            int count = mAllBalls.count();
            for (int i = 0; i < count; i++)
            {
                Ball testBall = mAllBalls.getItem(i);

                float testRadius = testBall.getRadius();
                Vector3 testCenter = testBall.getCenter();
                float testSpeed = testBall.getVelocity().Length();
                for (int j = i + 1; j < count; j++)
                {
                    Ball otherBall = mAllBalls.getItem(j);
                    float otherRadius = otherBall.getRadius();
                    Vector3 otherCenter = otherBall.getCenter();
                    Vector3 v = otherCenter - testCenter;
                    float vLength = v.Length();
                    if (vLength < (testRadius + otherRadius))
                    {
                        Vector3 normalizeV = v;
                        normalizeV.Normalize();
                        float otherSpeed = otherBall.getVelocity().Length();
                        testBall.setVelocity(-testSpeed * normalizeV);
                        otherBall.setVelocity(otherSpeed * normalizeV);
                    }
                }
            }

            float accelY = -15f * elapsedSeconds;

            for (int i = 0; i < count; i++)
            {
                Ball ball = mAllBalls.getItem(i);

                Vector3 velocity = ball.getVelocity();
                velocity.Y += accelY;

                ball.setVelocity(velocity);
            }

            for (int i = 0; i < count; i++)
            {
                Ball ball = mAllBalls.getItem(i);

                Vector3 updateVector = ball.getVelocity() * elapsedSeconds;
                Vector3 center = ball.getCenter() + updateVector;

                ball.setCenter(center);
            }

            for (int i = mAllBalls.count() - 1; i >= 0; i--)
            {
                Ball ball = mAllBalls.getItem(i);
                if(ball.isOutsideBounds(mWorldWidth, mWorldHeight))
                {
                    mAllBalls.deleteItem(i);

                    if(mSelectedIndex >= i)
                        mSelectedIndex--;
                    else if(mSelectedIndex == i)
                        mSelectedIndex = -1;
                }
            }
        }

        public void drawModel()
        {
            int count = mAllBalls.count();

            for (int i = 0; i < count; i++)
            {
                Ball ball = mAllBalls.getItem(i);
                if (i != mSelectedIndex)
                    ball.draw();
            }

            Ball selected = selectedBall();
            if (selected != null)
            {
                Color oldColor = selected.getFillColor();
                selected.setFillColor(new Color(255, 0, 0));
                selected.draw();
                selected.setFillColor(oldColor);
            }
        }
    }
}
