using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using UWBGL_WinForms_Lib1;
using UWBGL_XNA_Lib4;

namespace XNA_ObjectsAndAttributes
{
	public enum GraphicsObjectType
	{
		gotPoint,
		gotLine,
		gotCircle,
	};

	public class Model
	{
		private GraphicsObjectType mCurrentObjectType;

		private UWB_CircleGeom mCircle;
		private UWB_LineGeom mLine;
		private UWB_PointGeom mPoint;

        private UWB_Clock mStopWatch;
        private float mWorldWidth;
        private float mWorldHeight;

		private UWB_XNADrawHelper mDrawHelper;

        public Model()
        {
			mCurrentObjectType = GraphicsObjectType.gotPoint;
            mWorldWidth = 0;
            mWorldHeight = 0;

			mDrawHelper = new UWB_XNADrawHelper();

			mCircle = new UWB_CircleGeom();
			mCircle.setColor(Color.Black);
			mLine = new UWB_LineGeom();
			mPoint = new UWB_PointGeom();
            mStopWatch = new UWB_Clock();
        }

        public void setWorldSize(float width, float height)
        {
            mWorldWidth = width;
            mWorldHeight = height;
        }

        public void getWorldSize(ref float width, ref float height)
        {
            width = mWorldWidth;
            height = mWorldHeight;
        }

		public void setCurrentGraphicsType(GraphicsObjectType objectType)
		{
			mCurrentObjectType = objectType;
		}

		public void setObjectColor(Color color)
		{
			switch (mCurrentObjectType)
			{
			case GraphicsObjectType.gotPoint:
				mPoint.setColor(color);
				break;
			case GraphicsObjectType.gotLine:
				mLine.setColor(color);
				break;
			case GraphicsObjectType.gotCircle:
				mCircle.setColor(color);
				break;
			}
		}

		public void setObjectShadingColor(Color color)
		{
			if (mCurrentObjectType == GraphicsObjectType.gotLine)
				mLine.setShadingColor(color);
			else if (mCurrentObjectType == GraphicsObjectType.gotCircle)
				mCircle.setShadingColor(color);
		}

		public void setShadeMode(eShadeMode mode)
		{
			if (mCurrentObjectType == GraphicsObjectType.gotLine)
				mLine.setShadeMode(mode);
			else if (mCurrentObjectType == GraphicsObjectType.gotCircle)
				mCircle.setShadeMode(mode);
		}

		public eShadeMode getShadeMode()
		{
			if (mCurrentObjectType == GraphicsObjectType.gotLine)
				return mLine.getShadeMode();
			else if (mCurrentObjectType == GraphicsObjectType.gotCircle)
				return mCircle.getShadeMode();
			else
				return eShadeMode.smFlat;
		}

		public void setCircleFillMode(bool filled)
		{
			if (filled)
				mCircle.setFillMode(eFillMode.fmSolid);
			else
				mCircle.setFillMode(eFillMode.fmWireframe);
		}

		public bool setPointSize(float size)
		{
			if (size < 1 || size > 15)
				return false;
			mPoint.setPointSize(size);
			return true;
		}

		public void startDefineGraphicsObject(float worldX, float worldY)
		{
			switch (mCurrentObjectType)
			{
			case GraphicsObjectType.gotPoint:
				mPoint.setLocation(worldX, worldY, 0);
				break;
			case GraphicsObjectType.gotLine:
				mLine.setStartPoint(worldX, worldY, 0);
				mLine.setEndPoint(worldX, worldY, 0);
				break;
			case GraphicsObjectType.gotCircle:
				mCircle.setCenter(worldX, worldY, 0);
				mCircle.setRadius(0);
				break;
			}
		}

		public void endDefineGraphicsObject(float worldX, float worldY)
		{
			switch (mCurrentObjectType)
			{
			case GraphicsObjectType.gotPoint:
				mPoint.setLocation(worldX, worldY, 0);
				break;
			case GraphicsObjectType.gotLine:
				mLine.setEndPoint(worldX, worldY, 0);
				break;
			case GraphicsObjectType.gotCircle:
				Vector3 center = mCircle.getCenter();
				float dx = worldX - center.X;
				float dy = worldY - center.Y;
				float radius = (float)Math.Sqrt(dx*dx + dy*dy);
				mCircle.setRadius(radius);
				break;
			}
		}

		public void moveGraphicsObject(float worldX, float worldY)
		{
			switch (mCurrentObjectType)
			{
			case GraphicsObjectType.gotPoint:
				mPoint.setLocation(worldX, worldY, 0);
				break;
			case GraphicsObjectType.gotLine:
				mLine.setEndPoint(worldX, worldY, 0);
				break;
			case GraphicsObjectType.gotCircle:
				mCircle.setCenter(worldX, worldY, 0);
				break;
			}
		}

		public void updateSimulation()
		{
			//nothing to do right now
		}

		public void drawModel()
		{
			mCircle.draw(mDrawHelper);
			mLine.draw(mDrawHelper);
			mPoint.draw(mDrawHelper);
		}
    }
}
