using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using UWBGL_WinForms_Lib1;
using UWBGL_XNA_Lib7;

namespace XNA_ObjectCollections
{
	public enum eGraphicsObjectType
	{
		gotPoint,
		gotLine,
		gotCircle,
	};

	public class Model
	{
		private UWB_PrimitiveList mAllPrimitives;

		private UWB_Primitive mCurrentObject;
		private eGraphicsObjectType mNewObjectType;
		private float mNewObjectPointSize;
		private Color mNewObjectColor;
		private Color mNewObjectShadingColor;
		private eFillMode mNewObjectFillMode;
		private eShadeMode mNewObjectShademode;

		private UWB_Clock mStopWatch;

		private UWB_BoundingBox mWorldBounds;
		private bool mShowBoundingVolumes;

		private UWB_XNADrawHelper mDrawHelper;

		public Model()
		{
			mWorldBounds = new UWB_BoundingBox();

			mDrawHelper = new UWB_XNADrawHelper();

			mAllPrimitives = new UWB_PrimitiveList();

			setCurrentTypeToPoint();

			mStopWatch = new UWB_Clock();
		}

		public void setWorldSize(float width, float height)
		{
			mWorldBounds.setCorners(Vector3.Zero, new Vector3(width, height, 0));
		}

		public void getWorldSize(ref float width, ref float height)
		{
			width = mWorldBounds.width();
			height = mWorldBounds.height();
		}

		public void setCurrentTypeToPoint()
		{
			mNewObjectType = eGraphicsObjectType.gotPoint;
		}

		public void setCurrentTypeToLine()
		{
			mNewObjectType = eGraphicsObjectType.gotLine;
		}

		public void setCurrentTypeToCircle()
		{
			mNewObjectType = eGraphicsObjectType.gotCircle;
		}

		public void setObjectColor(Color color)
		{
			if(mCurrentObject != null)
				mCurrentObject.setFlatColor(color);
			mNewObjectColor = color;
		}

		public void setShadingColor(Color color)
		{
			if (mCurrentObject != null)
				mCurrentObject.setShadingColor(color);
			mNewObjectShadingColor = color;
		}

		public void setShadeMode(eShadeMode shadeMode)
		{
			if (mCurrentObject != null)
				mCurrentObject.setShadeMode(shadeMode);
			mNewObjectShademode = shadeMode;
		}

		public void setPointSize(float size)
		{
			if (mCurrentObject != null)
				mCurrentObject.setPointSize(size);
			mNewObjectPointSize = size;
		}

		public void setFillMode(eFillMode fillMode)
		{
			if (mCurrentObject != null)
				mCurrentObject.setFillMode(fillMode);
			mNewObjectFillMode = fillMode;
		}

		public eShadeMode getShadeMode()
		{
			return (mCurrentObject != null) ? mCurrentObject.getShadeMode() : mNewObjectShademode;
		}

		public eFillMode getFillMode()
		{
			return (mCurrentObject != null) ? mCurrentObject.getFillMode() : mNewObjectFillMode;
		}

		public float getPointSize()
		{
			return (mCurrentObject != null) ? mCurrentObject.getPointSize() : mNewObjectPointSize;
		}

		public void startDefineGraphicsObject(float worldX, float worldY)
		{
			switch (mNewObjectType)
			{
			case eGraphicsObjectType.gotPoint:
				mCurrentObject = new UWB_PrimitivePoint();
				break;
			case eGraphicsObjectType.gotLine:
				mCurrentObject = new UWB_PrimitiveLine();
				break;
			case eGraphicsObjectType.gotCircle:
				mCurrentObject = new UWB_PrimitiveCircle();
				break;
			default:
				mCurrentObject = null;
				break;
			}

			mCurrentObject.setPointSize(mNewObjectPointSize);
			mCurrentObject.setFillMode(mNewObjectFillMode);
			mCurrentObject.setShadeMode(mNewObjectShademode);
			mCurrentObject.setFlatColor(mNewObjectColor);
			mCurrentObject.setShadingColor(mNewObjectShadingColor);

			mCurrentObject.mouseDownVertex(0, worldX, worldY);
			mCurrentObject.setVelocity(Vector3.Zero);
			mAllPrimitives.append(mCurrentObject);
		}

		public void endDefineGraphicsObject(float worldX, float worldY)
		{
			if(mCurrentObject != null)
				mCurrentObject.mouseDownVertex(1, worldX, worldY);
		}

		public void moveGraphicsObject(float worldX, float worldY)
		{
			if(mCurrentObject != null)
				mCurrentObject.moveTo(worldX, worldY);
		}

		public void updateSimulation()
		{
			int count = mAllPrimitives.count();
			eLevelofDetail lod = mDrawHelper.getLod();

			for (int i = count - 1; i >= 0; i--)
			{
				UWB_Primitive primitive = mAllPrimitives.primitiveAt(i);
				if (primitive != null)
				{
					UWB_BoundingVolume boundingVolume = primitive.getBoundingVolume(lod);
					if (boundingVolume == null || !boundingVolume.intersects(mWorldBounds))
					{
						if (mCurrentObject == primitive)
							mCurrentObject = null;
						mAllPrimitives.deletePrimitiveAt(i);
					}
				}
			}

			count = mAllPrimitives.count();

			for (int i = 0; i < count; i++)
			{
				UWB_Primitive testPrimitive = mAllPrimitives.primitiveAt(i);
				if (testPrimitive == null)
					continue;

				UWB_BoundingVolume testVolume = testPrimitive.getBoundingVolume(lod);
				if (testVolume == null)
					continue;

				for (int j = i + 1; j < count; j++)
				{
					UWB_Primitive otherPrimitive = mAllPrimitives.primitiveAt(j);
					if (otherPrimitive == null)
						continue;

					UWB_BoundingVolume otherVolume = otherPrimitive.getBoundingVolume(lod);
					if (otherVolume != null && testVolume.intersects(otherVolume))
					{
						testPrimitive.collisionResponse(otherPrimitive, otherVolume.getCenter());
						otherPrimitive.collisionResponse(testPrimitive, testVolume.getCenter());
					}
				}

				float elapsedSeconds = mStopWatch.getSecondsElapsed();
				mAllPrimitives.update(elapsedSeconds);
			}
		}

		public void drawModel()
		{
			eLevelofDetail lod = mDrawHelper.getLod();
			mAllPrimitives.draw(lod, mDrawHelper);

			if (mShowBoundingVolumes)
			{
				Color color = new Color(255, 0, 0);
				mAllPrimitives.drawChildBoundingVolumes(lod, mDrawHelper, color);
			}
		}

		public void placeGraphicsObjectInMotion()
		{
			if (mCurrentObject == null)
				return;

			float x = UWB_Utility.UWB_RandomNumber(-30, 30);
			float y = UWB_Utility.UWB_RandomNumber(-30, 30);
			Vector3 randomVelocity = new Vector3(x, y, 0);

			mCurrentObject.setVelocity(randomVelocity);
		}

		public void setLevelOfDetail(eLevelofDetail lod)
		{
			mDrawHelper.setLod(lod);
		}

		public eLevelofDetail getLevelOfDetail()
		{
			return mDrawHelper.getLod();
		}

		public void showBoundingVolumes(bool on)
		{
			mShowBoundingVolumes = on;
		}
	}
}

